open Unix;;
open Mis;;
open Misdir;;
open Logger;;
open Str;;

module Cdchunks =
	struct

exception IgnoreMe;;

let chunksize = 629145600;;
let outfilebase = "cdchunk";;

let fnamedot = Str.regexp "^.*\\.$";;
let fnamedotdot = Str.regexp "^.*\\.\\.$";;
let fnamecdchunk = Str.regexp "^cdchunk[0-9]+$";;

let dontskip name =
	((Str.string_match fnamedot name 0) == false) &&
	((Str.string_match fnamedotdot name 0) == false) &&
	((Str.string_match fnamecdchunk name 0) == false)
;;

let rec processFiles flist currentchunk chan cdsize cdcount totalsize totalcount =
	match flist with
			first :: rest ->

				let size = Misdir.fsize first in
					if cdsize + size > chunksize then
						begin
							print_endline ("Closing chunk file, opening new one, cdsize=" ^ (string_of_int cdsize)
								^ ", cdcount=" ^ (string_of_int cdcount)
								^ ", totalsize=" ^ (Big_int.string_of_big_int totalsize));
							close_out !chan;
							currentchunk := !currentchunk + 1;
							chan := open_out (outfilebase ^ (string_of_int !currentchunk));
							output_string !chan (first ^ "\n");
							processFiles rest currentchunk chan size (1) (Big_int.add_int_big_int size totalsize) (totalcount+1)
						end
					else
						begin
							output_string !chan (first ^ "\n");
							processFiles rest currentchunk chan (cdsize + size) (cdcount+1) (Big_int.add_int_big_int size totalsize) (totalcount+1)
						end

		| [] ->
							print_endline ("Closing chunk file, opening new one, cdsize=" ^ (string_of_int cdsize)
								^ ", cdcount=" ^ (string_of_int cdcount)
								^ ", totalsize=" ^ (Big_int.string_of_big_int totalsize));
				print_endline "successful completion..."
;;

let usage = "Usage: cdchunks.ml -directory <directoryName> -tracelevel <#> -skip <dir:dir:dir>";;

let dname = ref "";;
let skipstr = ref "";;
let tlevel = ref 0;;

Arg.parse [
  Mis.argsd "-directory" dname "directory name.";
	Mis.argi "-tracelevel" tlevel "tracelevel, 4 for all.";
	Mis.argsd "-skip" skipstr "directories to skip.";
] (fun y -> Printf.printf "%s\n" y) usage;

Logger.setchan stdout;;
if !tlevel != 0 then
	Logger.setTraceLevel !tlevel;;

try
	let flist = ref [] in
		Misdir.getFilesystemNames !dname [Misdir.Regular;] (Str.split (Str.regexp ":") !skipstr) flist;

		let chan = ref (open_out (outfilebase ^ (string_of_int 0))) in
		let currentchunk = ref 0 in
			print_endline ("calling processFiles with list of " ^ (string_of_int (List.length !flist)) ^ " files.");
			processFiles !flist currentchunk chan 0 0 Big_int.zero_big_int 0

with x ->
	print_endline "Exception caught...";
	print_endline (Printexc.to_string x)
;;

	end
;;
