open Mis
module Unix = UnixLabels

module Examiner =
  struct

exception IoFailure;;

class requestDescriptor =
  object (self)
    val mutable rid = ""
    val mutable host = ""
    val mutable examiningUrl = ""
    val mutable complete = false
    val mutable accept = false
    method setrid r = rid <- r
    method getrid = rid
    method sethost h = host <- h
    method gethost = host
    method setcomplete = complete <- true
    method setaccept = accept <- true
    method asString = "rid=" ^ rid ^
					  ",host=" ^ host ^
					  ",url=" ^ examiningUrl ^
					  "complete=" ^ string_of_bool complete ^
					  "accept=" ^ string_of_bool accept
    method url = if complete
    then
      "http://" ^ host
    else
      examiningUrl;
  end
;;

class requestQueue =
  object
    val mucv = new Mis.mucv;
    val requests = Queue.create();
    method add (req:requestDescriptor) =
      mucv#lock;
      Queue.add req requests;
      mucv#unlock;
    method get =
      mucv#lock;
      while Queue.length requests == 0 do mucv#wait done;
      let req = Queue.take requests in
        mucv#unlock;
        req;
  end
;;

let handleConnection ((cfd:Unix.file_descr),(csaddr:Unix.sockaddr),(rq:requestQueue)) =
  let req = rq#get in
  let buf = String.create 2048 in
  let len = Marshal.to_buffer buf 0 (String.length buf) req [] in
    ignore(Unix.write cfd buf 0 len)
;;

type arg_type = requestQueue;;

let arg = new requestQueue;;

(*end of module*)
end
;;
