

module Misdir =
	struct
		type filetype = Regular | Directory | Link | Socket | Fifo | CharDevice | BlockDevice;;

		exception InvalidFiletype;;

		let whichftype lstr =
			match lstr with
					"Regular" -> Regular
				| "Directory" -> Directory
				| "Link" -> Link
				| "Socket" -> Socket
				| "Fifo" -> Fifo
				| "CharDevice" -> CharDevice
				| "BlockDevice" -> BlockDevice
				| _ -> raise InvalidFiletype;;

		let rec ftypelist_from_stringlist stlist tlist =
			match stlist with
					first :: rest -> ftypelist_from_stringlist rest ((whichftype first)::tlist)
				| [] -> tlist
		;;

		let ftype name =
			let st = Unix.lstat name in
			let kind = st.Unix.st_kind in
				match kind with
						Unix.S_REG -> Regular
					| Unix.S_DIR -> Directory
					| Unix.S_CHR -> CharDevice
					| Unix.S_BLK -> BlockDevice
					| Unix.S_LNK -> Link
					| Unix.S_FIFO -> Fifo
					| Unix.S_SOCK -> Socket
		;;

		let fsize name =
			let st = Unix.lstat name in
				st.Unix.st_size
		;;

		let reDots = Str.regexp "^.*\\.\\(\\.\\)?$";;

		let makeName dname fname =
			dname ^ "/" ^ fname

		let rec inskips dname skips =
			match skips with
					first :: rest ->
						if Str.string_match (Str.regexp first) dname 0
						then true
						else inskips dname rest

				| [] -> false;;

		let rec getFilesystemNames dname types skips dlist =
			let dhandle = Unix.opendir dname in
			let flag = ref true in
				while !flag do
					try
						let fname = (makeName dname (Unix.readdir dhandle)) in
							if (Str.string_match reDots fname 0) == false && (inskips fname skips) == false then
								let ty = ftype fname in
									if (List.mem ty types) then
											dlist := fname::!dlist;
									if ty == Directory then
										getFilesystemNames fname types skips dlist
					with End_of_file ->
						Unix.closedir dhandle;
						flag := false
				done
		;;
	end
;;
