open Mis;;
module Unix = UnixLabels;;
module Stringmap = Map.Make (String);;
module Stringset = Set.Make (String);;
open Printf;;
open Str;;
open List;;

module Urldb =
	struct
		type sitestatus = Black | White | Unknown;;
		exception InvalidSiteType;;

		let sitestatusAsString sstatus =
			match sstatus with
					Black -> "black"
				| White -> "white"
				| Unknown -> "unknown";;

		class urldb clientid =
			object(self)
				val mutable dbmap = Stringmap.empty;
				val mutable dbset = Stringset.empty;

				initializer
					(*site \t [black|white]*)
					let chan = open_in_gen [Open_rdonly;Open_text;] 0 clientid in
						begin
							try
								while true do
									let li = input_line chan in
									let fs = Str.split (regexp "\t") li in
									let siteid = (List.nth fs 0) in
										dbset <- (Stringset.add siteid dbset);
										match (List.nth fs 1) with
												"black" -> dbmap <- Stringmap.add siteid Black dbmap
											| "white" -> dbmap <- Stringmap.add siteid White dbmap
											| _ -> raise InvalidSiteType
								done
							with End_of_file ->
								print_endline "data input complete";
						end;
						close_in chan;

				method status site =
					try
						match (Stringmap.find site dbmap) with
								Black -> Black
							| White -> White
							| Unknown -> Unknown
						with Not_found ->
							Unknown;

				method showentry key data = Printf.printf "line: %s.%s\n" key data;

				method displayOne keylist =
					match keylist with
							first :: rest -> self#showentry first (sitestatusAsString (Stringmap.find first dbmap))
						| [] -> ignore();

				method dumplist =
					let el = Stringset.elements dbset in
						print_string "in dumplist, list size=";
						print_int (List.length el);
						print_newline();
						self#displayOne el;
		end

	end;;
