open Examiner
open Tcpserver
open Client
open Mis
open Logger
open Htmlimage

module ExaminerRequestServer = Tcpserver(Examiner)
module ClientRequestServer = Tcpserver(Client)

module Webserver =
  struct

	let port = ref 9999 in
	let tracelevel = ref 0 in
	let logname = ref "wserverlog" in
	let tracename = ref "wtracelog" in
	let blpage = ref "BlacklistPage" in 
	let ckpage = ref "CheckingsitePage" in
	let headerlog = ref "wsHeaderLog" in
	let responselog = ref "wsResponseLog" in
	let htmlBlockerRoot = ref "/usr/local/apache2/blocker/htdocs" in
	let htmlExaminerRoot = ref "/usr/local/apache2/htdocs" in
	let redirectHeader = ref "redirect.header" in
	let examinerHtml = ref "examiner.html" in
	let examiningHtml = ref "examining.html" in
	let examinerServerName = ref "http://amethyst.cql.com/" in
	let cgiport = ref 12000 in

	let usage = "Usage:\n-port <port#>\n-tracelevel <#>\n-logname <file>\n-tracename <file>\n..." in

		try
			print_endline "before Arg.parse";

			Arg.parse [
				Mis.argi "-port" port "port number";
				Mis.argi "-tracelevel" tracelevel  "tracing level";
				Mis.argsd "-logname" logname "log file name";
				Mis.argsd "-tracename" tracename "tracefile name";
				Mis.argsd "-blpage" blpage "blacklist HTML template";
				Mis.argsd "-ckpage" ckpage "checking site template";
				Mis.argsd "-headerlog" headerlog "header log file name";
				Mis.argsd "-responselog" responselog "response log file name";
				Mis.argsd "-htmlBlockerRoot" htmlBlockerRoot "block htdocs";
				Mis.argsd "-htmlExaminerRoot" htmlExaminerRoot "htdocs directory for examiner";
				Mis.argsd "-redirectHeader" redirectHeader "template for returning redirect header";
				Mis.argsd "-examinerHtml" examinerHtml "template for examiner display";
				Mis.argsd "-examiningHtml" examiningHtml "template for client examining display";
				Mis.argsd "-examinerServerName" examinerServerName "server for examiner displays";
				Mis.argi "-cgiport" cgiport "port for examiner contacts";
			] (fun y -> Printf.printf "%s\n" y)  usage;

			Logger.setTraceLevel !tracelevel;
			Logger.setchanfromoutchan stdout;
			Logger.setexchanfromoutchan stderr;

			Logger.show 3 "webserver.ml" 57 "main...";

			let rheader = new Htmlimage.memoryHtmlImage !redirectHeader in
			let rqueue = new Examiner.requestQueue in
			let tid1 = ExaminerRequestServer.start "192.168.1.14" 12000 in
			let tid2 = ClientRequestServer.start "192.168.1.14" 9999 in
				Thread.join tid1;
				Thread.join tid2;

		with x ->
			Logger.showex "webserver.ml" 67 x "exception at end of main()";

	(*module end*)
	end
